
local addon, AR = ...;
local L=AR.L;
local autoConfirm = {};
local version=0.92;
AR.ModeLabels = {GREED,NEED,ROLL_DISENCHANT,PASS,L["Do nothing"]};
AR.AutoRollDB_defaults = {
	enabled = true,
	autoConfirmAll=false,
	version = version,
	items = {
		[120945]=1, -- Primal Spirit
		[116920]=5, -- True steel lockbox
	}
};
AutoRollDB = AR.AutoRollDB_defaults;

local itemNames = {};
local function updateItemNames()
	wipe(itemNames);
	for i,v in pairs(AutoRollDB.items)do
		local name=GetItemInfo(i);
		if(name)then
			itemNames[name] = v;
		end
	end
end
AR.updateItemNames=updateItemNames;

--- addon setup
local main = CreateFrame("Frame", "AutoRollFrame");

main:RegisterEvent("ADDON_LOADED");

main:SetScript("OnEvent", function(self, event, arg1, arg2)
	if(event=="ADDON_LOADED" and addon==arg1)then
		print("|Cff3388ff"..addon,"|Cffffff00"..L["Addon loaded..."].."|r");
		print("|cff3388ff"..addon,"|cffffff00"..L["Addon"],(AutoRollDB.enabled and L["enabled"] or L["disabled"]) .. "...|r");
		if(AutoRollDB.version==nil)then AutoRollDB.version=0; end
		if(AutoRollDB.version<=version)then
			for i,v in pairs(AR.AutoRollDB_defaults.items)do
				if(AutoRollDB.items[i]==nil)then
					AutoRollDB.items[i] = v;
				end
			end
		end
		AutoRollDB.version=version;
		if(AutoRollDB.enabled)then
			self:RegisterEvent("START_LOOT_ROLL");
			self:RegisterEvent("CONFIRM_LOOT_ROLL");
			updateItemNames();
		end
	end

	-- auto need on certain items
	if(event=="START_LOOT_ROLL")then
		-- search the lootframes for the item and click "need"
		for i = 1, NUM_GROUP_LOOT_FRAMES do
			-- Currently ElvUI is unsupportable. (ElvUI's own lootFrame objects have no framenames or connection to its parent (ElvUIParent) like parentKeys)
			local _, name, _, _, _, canNeed, canGreed, canDisenchant, reasonNeed, reasonGreed, reasonDisenchant = GetLootRollItemInfo(arg1)
			local lootFrame = _G["GroupLootFrame"..i];
			if(lootFrame:IsVisible() and itemNames[name])then
				if(not AutoRollDB.autoConfirmAll) and (reasonNeed or reasonGreed or reasonDisenchant)then
					autoConfirm[arg1]=true;
				end
				local button = nil;
				if canNeed and itemNames[name]==2 then
					button = "NeedButton";
				elseif canDisenchant and itemNames[name]==3 then
					button = "DisenchantButton";
				elseif canGreed and (itemNames[name]==1 or itemNames[name]==2 or itemNames[name]==3) then
					--- with fallback if need or disenchant not possible
					button = "GreedButton";
				elseif itemNames[name]==4 then
					button = "PassButton";
				end
				if(button)then
					lootFrame[button]:Click();
				end
			end
		end
	end

	-- confirm roll automatical
	if(event=="CONFIRM_LOOT_ROLL")then
		if(autoConfirm[arg1] or AutoRollDB.autoConfirmAll==true)then
			ConfirmLootRoll(arg1, arg2);
			autoConfirm[arg1]=nil;
		end
	end
end)

--- command line options
SlashCmdList["AUTOROLL"] = function(cmd)
	local cmd, arg = strsplit(" ", cmd:lower(), 2)

	local n, name = tonumber(arg);

	if(n)then
		name=GetItemInfo(n);
	end

	print("|Cff0088ff"..addon.."|r");
	if(cmd=="")then
		print(L["Chat command list for /ar & /autoroll"]);
		print(L["toggle - enable/disable AutoRoll's functionality"]);
		print(L["autoconfirmall - enable/disable AutoConfirm on all items."]);
		print(L["list - list items"]);
		print(L["add <itemId> - add an item to the item list"]);
		print(L["del <itemId> - delete an item from the item list"]);
		print(L["mode <itemId> - change need/greed/disenchant/pass/nothing mode for a single item"]);
		print(L["reset - reset the item list to default"]);

	elseif(cmd=="toggle")then
		AutoRollDB.enabled = not AutoRollDB.enabled;
		print(L["AutoRoll's functionality"] .. " " .. (AutoRollDB.enabled and L["enabled"] or L["disabled"]));

	elseif(cmd=="autoconfirmall")then
		AutoRollDB.autoConfirmAll = not AutoRollDB.autoConfirmAll;
		print(L["AutoConfirm all"] .. " " .. (AutoRollDB.autoConfirmAll and L["enabled"] or L["disabled"]));

	elseif(cmd=="list")then
		print(L["Current item list"]);
		for i,v in pairs(AutoRollDB.items)do
			local name=GetItemInfo(i);
			print(L["Id: %d (%s), Mode: %s"]:format(i,name,AR.ModeLabels[v] or "?"));
		end

	elseif(cmd=="add" and n~=nil and n>0 and not AutoRollDB.items[n])then
		local name=GetItemInfo(n);
		if(name)then
			AutoRollDB.items[n]=1;
			if(AutoRollPanel and AutoRollPanel:IsShown())then
				AutoRollPanel.changes.items[n]=1;
			end
			updateItemNames();
			print(L["ItemID %d found... %s added to AutoRoll's item list. Mode: %s"]:format(n,name,GREED));
		else
			print(L["ItemID %d unknown/invalid... please check the given numbers."]:format(n));
		end

	elseif(cmd=="del")then
		if(n~=nil and AutoRollDB.items[n])then
			AutoRollDB.items[n] = nil;
			if(AutoRollPanel and AutoRollPanel:IsShown())then
				AutoRollPanel.changes.items[n]=nil;
			end
			updateItemNames();
			print(L["ItemID %d (%s) removed from AutoRoll's item list"]:format(n,name));
		else
			print(L["Id %d not found in item list."]:format(n));
		end

	elseif(cmd=="mode")then
		if(n and n>0 and AutoRollDB.items[n])then
			AutoRollDB.items[n] = AutoRollDB.items[n]==#AR.ModeLabels and 1 or AutoRollDB.items[n]+1;
			if(AutoRollPanel and AutoRollPanel:IsShown())then
				AutoRollPanel.changes.items[n]=AutoRollDB.items[n];
			end
			print(L["Changed mode for itemId %d (%s) to %s"]:format(n, name, AR.ModeLabels[AutoRollDB.items[n]]));
		else
			print(L["ItemId %d not found in item list."]:format(n));
		end

	elseif(cmd=="reset")then
		AutoRollDB = AR.AutoRollDB_defaults;
		updateItemNames();
		if(AutoRollPanel and AutoRollPanel:IsShown())then AutoRollPanel:refresh(); end
		print(L["Item list resetted..."]);
	end

	if(AutoRollPanel and AutoRollPanel:IsShown())then AutoRollPanel:refresh(true); end
end

SLASH_AUTOROLL1 = "/autoroll"
SLASH_AUTOROLL2 = "/ar"

