
local addon, AR, panel = ...;
local L=AR.L;

---------------------------------------------------------------
---------------------------------------------------------------

local function getValue(a,b)
	if(b)then
		return (panel.changes[a][b]~=nil) and panel.changes[a][b] or AutoRollDB[a][b];
	end
	return (panel.changes[a]~=nil) and panel.changes[a] or AutoRollDB[a];
end

local function setValue(a,b,c)
	if(c)then
		if(panel.changes[a]==nil)then panel.changes[a] = {}; end
		panel.changes[a][b] = c;
	else
		panel.changes[a] = b;
	end
end

local function setControl(frame,fnc)
	if not (frame.GetObjectType) then return end
	frame.ctrl = fnc;
	table.insert(panel.controls,frame);
end

local function setTooltip(frame,...)
	if(select('#',...)==0)then return end
	frame.tooltip = {...};
	frame:SetScript("OnLeave",function() GameTooltip:Hide(); end);
	frame:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT");
		GameTooltip:AddLine(self.tooltip[1]);
		for i=2, #self.tooltip do
			if(type(self.tooltip[i])=="table")then
				GameTooltip:AddLine(unpack(self.tooltip[i]));
			else
				GameTooltip:AddLine(self.tooltip[i],1,1,1,1);
			end
		end
		GameTooltip:Show()
	end);
end

local function setPoints(frame,...)
	frame = (type(frame)=="string" and panel[frame]) or (type(frame)=="table" and frame) or nil;
	if(not frame)then return end

	local point1=...;
	if(point1==true)then
		frame:SetAllPoints();
	elseif(#{...}>0)then
		for _,point in ipairs({...})do
			frame:SetPoint(unpack(point));
		end
	else
		frame:SetPoint("TOPLEFT");
	end
end

---------------------------------------------------------------
---------------------------------------------------------------

local itemIds = {};
local function updateItemList(self)
	local offset = HybridScrollFrame_GetOffset(self);

	if(not panel.changes.items)then
		panel.changes.items = CopyTable(AutoRollDB.items);
		wipe(itemIds);
	end
	if(#itemIds==0)then
		for i,v in pairs(panel.changes.items)do
			if(v~=nil)then
				table.insert(itemIds,i);
			end
		end
		table.sort(itemIds,function(a,b) return a>b; end);
	end
	local nItems,items = #itemIds,panel.changes.items;

	for i=1, #self.buttons do
		local index = offset+i;
		local button = self.buttons[i];

		if(itemIds[index])then
			local Id = itemIds[index];
			local Name, _, Rarity, _, _, _, _, _, _, Texture = GetItemInfo(Id);

			button.ItemID:SetText(Id);
			button.ItemIcon:SetTexture(Texture);
			button.ItemName:SetText(Name);
			button.ItemName:SetPoint("RIGHT",button.Mode,"LEFT",-10,0); -- limit width of font string

			button.Mode.index = Id;
			button.Mode.value = items[Id];

			UIDropDownMenu_SetWidth(button.Mode, 94);
			UIDropDownMenu_Initialize(button.Mode, function()
				for v,t in pairs(AR.ModeLabels) do
					local info = UIDropDownMenu_CreateInfo();
					info.value = v; info.text = t;
					info.arg1 = button.Mode; info.arg2 = v;
					info.func = function(self,frame,value)
						UIDropDownMenu_SetSelectedValue(frame, value);
						panel.changes.items[button.Mode.index]=value;
						button.Mode.value=value;
					end
					UIDropDownMenu_AddButton(info);
				end
			end);
			UIDropDownMenu_SetSelectedValue(button.Mode, items[Id]);

			local b=_G[button.Mode:GetName().."Button"];
			b:SetHitRectInsets(-(button.Mode:GetWidth()-(b:GetWidth()*2)),-3,-3,-3)

			button.Delete:SetScript("OnClick",function()
				panel.changes.items[Id] = nil;
				wipe(itemIds);
				updateItemList(self);
			end);

			button:SetBackdropColor(1, 1, 1, (index/2==floor(index/2)) and 0 or .1 );

			button:Show();
		else
			button:Hide();
		end
	end

	HybridScrollFrame_Update(self, nItems * self.buttonHeight, #self.buttons * self.buttonHeight);
end

local function checkInput()
	panel.AddNewItem:Disable();
	panel.ItemHelper:Hide();

	local obj,itemName,itemLink,itemTexture,itemId = panel.InputNewItem:GetText();
	if(obj)then
		itemName,itemLink,_,_,_,_,_,_,_,itemTexture = GetItemInfo(obj);
	end

	if(itemName and itemLink)then
		itemId = itemLink:match("Hitem:(%d+)");
		if(not itemId) or (itemId and (panel.changes.items[itemId]~=nil or AutoRollDB.items[itemId]~=nil))then
			itemName, itemId = nil, nil;
		end
	end

	if(itemId and itemName)then
		panel.InputNewItem.itemID = itemId;
		panel.ItemHelper:SetText( ("|T%s:24:24:0:0|t %s"):format(itemTexture,itemName) );
		panel.ItemHelper:Show();
		panel.AddNewItem:Enable();
	end
end

local function addItem()
	local id = tonumber(panel.InputNewItem.itemID);
	if(id and not AutoRollDB.items[id] and not panel.changes.items[id])then
		panel.changes.items[id] = 1; -- greed is default
		panel.InputNewItem:SetText("");
		panel:refresh(true);
	end
end

---------------------------------------------------------------
---------------------------------------------------------------

panel = CreateFrame("Frame", "AutoRollPanel", InterfaceOptionsFramePanelContainer);
panel:SetAllPoints();
panel:Hide();
panel.name, panel.controls, panel.changes, panel.temp = addon, {}, {}, {};

function panel:okay()
	for i,v in pairs(panel.changes)do
		if(i=="items")then
			AutoRollDB.items = CopyTable(panel.changes.items);
		else
			AutoRollDB[i] = v;
		end
	end
	panel.changes={};
	AR.updateItemNames();
end

function panel:cancel()
	panel.changes={};
end

function panel:default()
	AutoRollDB = AR.AutoRollDB_defaults;
	ReloadUI();
end

function panel:refresh(force)
	if(force==true)then
		wipe(itemIds);
	end
	for _,frame in pairs(panel.controls)do
		if(type(frame.ctrl)=="function")then
			frame.ctrl();
		end
	end
end

panel.checkInput = false;
panel.elapsed=0;
panel:SetScript("OnUpdate", function(self,elapse)
	if(self.checkInput)then
		self.elapsed = self.elapsed + elapse;
		if(self.elapsed>.2)then
			self.checkInput=false;
			checkInput();
		end
	end
end);

panel:SetScript("OnEvent",function(self,event,...)
	if(self.InputNewItem and self.InputNewItem:HasFocus())then
		checkInput();
	end
end);
panel:RegisterEvent("GET_ITEM_INFO_RECEIVED");

panel:SetScript("OnShow", function()
	--- Title
	panel.title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	setPoints(panel.title,{"TOPLEFT", panel, 12, -12});
	panel.title:SetText(addon.. " - "..OPTIONS);

	--- Subtitle
	panel.subtitle = panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	setPoints(panel.subtitle,{"TOPLEFT", panel.title, "BOTTOMLEFT", 0, -4});
	panel.subtitle:SetText(("Version: %s / Author: Hizuro"):format(GetAddOnMetadata(addon, "Version")));

	--- Credit
	panel.credit = panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
	setPoints(panel.credit,{"BOTTOMLEFT", panel, "BOTTOMLEFT", 12, 12});
	panel.credit:SetText(L["Credit: Azial for initial idea and first piece of code in german WoW forum."]);

	--- Checkbutton "Enable <addon>"
	local n,N="EnableAR";
	N=panel:GetName()..n;
	panel[n]=CreateFrame("CheckButton",N,panel,"OptionsCheckButtonTemplate");
	setPoints(panel[n],{"TOPLEFT",panel,"TOPRIGHT",-160,-6});
	_G[N.."Text"]:SetText(L["Enable AutoRoll"]);
	--panel[n]:SetChecked(getValue("enabled")==true);
	panel[n]:SetScript("OnClick",function() setValue("enabled",not getValue("enabled")); end);
	setControl(panel[n],function() panel[n]:SetChecked(getValue("enabled")); end);
	setTooltip(panel[n],L["Enable AutoRoll"],L["Enable/Disable automatic rolling on listed items."]);

	--- Checkbutton "AutoConfirm all"
	local n,N="AutoConfirmAll";
	N=panel:GetName()..n;
	panel[n]=CreateFrame("CheckButton",N,panel,"OptionsCheckButtonTemplate");
	setPoints(panel[n],{"TOPLEFT", panel.EnableAR, "BOTTOMLEFT", 0, 2});
	_G[N.."Text"]:SetText(L["AutoConfirm all"]);
	--panel[n]:SetChecked(getValue("autoConfirmAll")==true);
	panel[n]:SetScript("OnClick",function() setValue("autoConfirmAll",not getValue("autoConfirmAll")); end);
	setControl(panel[n],function() panel[n]:SetChecked(getValue("autoConfirmAll")); end);
	setTooltip(panel[n],L["AutoConfirm all"],L["Automaticly confirm all group roll requests"],{L["Not recommented"],1,.5,0},{L["Default:"].." "..L["Disabled"],.6,.6,.6});

	--- Inset border frame for itemList (scrollFrame)
	local n, N, b="itemList"; N, b=panel:GetName()..n, n.."Border";
	panel[b] = CreateFrame("Frame", nil, panel);
	setPoints(panel[b],{"TOPLEFT", panel.title, "BOTTOMLEFT", 0, -26},{"BOTTOMRIGHT", -14, 105});
	panel[b]:SetBackdrop({ bgFile="Interface\\TutorialFrame\\TutorialFrameBackground", edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", tile=true, tileSize=32, edgeSize=20, insets={left=4,right=4,top=4,bottom=4}});

	--- itemList ScrollFrame
	panel[n] = CreateFrame("ScrollFrame",N,panel[b],"BasicHybridScrollFrameTemplate");
	--panel[b]:SetBackdrop({ bgFile="Interface\\Buttons\\white8x8", tile=true });
	setPoints(panel[n],{"TOPLEFT",8,-7},{"BOTTOMRIGHT",-28,7});
	setPoints(panel[n].scrollBar,{"TOPLEFT",panel[n],"TOPRIGHT", 0, -14},{"BOTTOMLEFT",panel[n],"BOTTOMRIGHT", 0, 13});
	panel[n].scrollBar.doNotHide=true;
	panel[n].update = updateItemList;
	HybridScrollFrame_CreateButtons(panel[n],"AutoRoll_ItemButtonTemplate",0,0,nil,nil,0,-3);
	setControl(panel[n],function() updateItemList(panel[n]); end);
	
	--- add new items
	local n="InputNewItem";
	panel[n] = CreateFrame("EditBox", nil, panel, "InputBoxTemplate");
	panel[n]:SetSize(240, 26); panel[n]:SetAutoFocus(false);
	panel[n].Label = panel[n]:CreateFontString(nil, "ARTWORK", "GameFontNormal");
	panel[n].Label:SetPoint("BOTTOMLEFT", panel[n], "TOPLEFT");
	panel[n].Label:SetText(L["Add new item:"]);
	panel[n]:SetScript("OnTextChanged",function()
		panel.checkInput=true;
		panel.elapsed=0;
	end);
	panel[n]:SetScript("OnEnterPressed",addItem);

	setTooltip(panel[n],
		L["Add new Item"],
		L["Adding an item by its item id."],
		{L["Add an item by name is possible but it works not any time. Reason unknown."],1,1,0,1},
		{L["Press enter or push the Add-Button to add the item to the item list"],0,.8,.8,1}
	);
	setPoints(panel[n],{"TOPLEFT", panel.itemList, "BOTTOMLEFT", 20, -22});
	setControl(panel[n],function() panel[n]:SetText(""); end);

	--- little helper to show if item name or id correct.
	panel.ItemHelper = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	panel.ItemHelper:SetPoint("TOPLEFT",panel.InputNewItem,"BOTTOMLEFT", -3, -3);
	panel.ItemHelper:Hide();

	--- "add new item"-Button
	local n,N = "AddNewItem"; N=panel:GetName()..n;
	panel[n] = CreateFrame("Button", N, panel, "UIPanelButtonTemplate");
	panel[n].Text = _G[N.."Text"];
	panel[n].Text:SetText(ADD);
	panel[n]:SetWidth(panel[n].Text:GetWidth()+24);
	panel[n]:SetScript("OnClick",addItem);
	panel[n]:Disable();
	setPoints(panel[n],{"LEFT", panel.InputNewItem, "RIGHT", 10, 1});

	panel:refresh();
	panel:SetScript("OnShow",panel.refresh);
end);

InterfaceOptions_AddCategory(panel);
